/*
The "oslevent" event-handling library.  This is a library for
sequencing user-defined operations.

This is the only implementation file for the oslevent library.

Orion Sky Lawlor, olawlor@acm.org, 2005/1/28 (Public Domain)
*/
#include "oslevent.h"
#include <queue> /* C++ STL header */

/******** oslevent_class  *********/
oslevent_class::~oslevent_class() {}

/******* oslevent_pool *********/

/** The oslevent_pool implementation is just an STL queue. 
For prioritized execution, you'd need a std::prioqueue.
*/
class oslevent_pool_queue {
public:
	std::queue<oslevent_class *> events;
};

/**
  Add this event to the pool.  It will run during
  the next call to loop.
*/
void oslevent_pool::add(oslevent_class *evt)
{
	q->events.push(evt); /* add the event to the queue */
}

/**
  Process all events in the pool until the pool is empty.
*/
void oslevent_pool::loop(void)
{
	while (!q->events.empty())
	{
		/* Grab an event from the front of the queue */
		oslevent_class *evt=q->events.front();
		q->events.pop(); /* remove it from the queue */
		/* Execute the event */
		evt->execute();
	}
}

oslevent_pool::oslevent_pool() /* constructor: make an empty pool */
{
	q=new oslevent_pool_queue;
}

oslevent_pool::~oslevent_pool() /* destructor: clean out the pool */
{
	delete q;
}

